<?php

namespace Lc\SovBundle\Controller\Ticket;

use Doctrine\ORM\EntityManagerInterface;
use EasyCorp\Bundle\EasyAdminBundle\Collection\FieldCollection;
use EasyCorp\Bundle\EasyAdminBundle\Collection\FilterCollection;
use EasyCorp\Bundle\EasyAdminBundle\Config\Action;
use EasyCorp\Bundle\EasyAdminBundle\Config\Actions;
use EasyCorp\Bundle\EasyAdminBundle\Config\Assets;
use EasyCorp\Bundle\EasyAdminBundle\Config\Crud;
use EasyCorp\Bundle\EasyAdminBundle\Context\AdminContext;
use EasyCorp\Bundle\EasyAdminBundle\Dto\EntityDto;
use EasyCorp\Bundle\EasyAdminBundle\Dto\SearchDto;
use EasyCorp\Bundle\EasyAdminBundle\Field\AssociationField;
use EasyCorp\Bundle\EasyAdminBundle\Field\ChoiceField;
use EasyCorp\Bundle\EasyAdminBundle\Field\DateField;
use EasyCorp\Bundle\EasyAdminBundle\Field\DateTimeField;
use EasyCorp\Bundle\EasyAdminBundle\Field\IntegerField;
use EasyCorp\Bundle\EasyAdminBundle\Field\TextField;
use EasyCorp\Bundle\EasyAdminBundle\Router\AdminUrlGenerator;
use Lc\SovBundle\Container\Ticket\TicketContainer;
use Lc\SovBundle\Container\Ticket\TicketMessageContainer;
use Lc\SovBundle\Definition\ActionDefinition;
use Lc\SovBundle\Factory\Ticket\TicketFactory;
use Lc\SovBundle\Factory\Ticket\TicketFactoryInterface;
use Lc\SovBundle\Factory\Ticket\TicketMessageFactory;
use Lc\SovBundle\Factory\Ticket\TicketMessageFactoryInterface;
use Lc\SovBundle\Form\Ticket\TicketAdminFormType;
use Lc\SovBundle\Form\Ticket\TicketFormType;
use Lc\SovBundle\Form\Ticket\TicketMessageFormType;
use Lc\SovBundle\Form\Ticket\TicketStatusType;
use Lc\SovBundle\Model\Ticket\TicketInterface;
use Lc\SovBundle\Controller\AbstractAdminController;
use Lc\SovBundle\Model\Ticket\TicketModel;
use Lc\SovBundle\Repository\RepositoryQueryInterface;
use Lc\SovBundle\Translation\TranslatorAdmin;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\RequestStack;

abstract class TicketAdminController extends AbstractAdminController
{

    public function getRepositoryQuery() :RepositoryQueryInterface
    {
        return $this->getTicketContainer()->getRepositoryQuery();
    }

    public function createEntity(string $entityFqcn)
    {
        return $this->getTicketContainer()->getFactory()->create();
    }

    public function configureCrud(Crud $crud): Crud
    {
        $crud =  parent::configureCrud($crud); // TODO: Change the autogenerated stub
        $crud->setDefaultSort(array('updatedAt'=> 'DESC'));
        return $crud;
    }

    public function configureAssets(Assets $assets): Assets
    {
        $assets = parent::configureAssets($assets);

        $assets->addWebpackEncoreEntry('sov-ticket');

        return $assets;
    }

    public function configureFields(string $pageName): iterable
    {
        return $this->getTicketContainer()->getFieldDefinition()->getFields($pageName);
    }

    public function configureActions(Actions $actions): Actions
    {
        $actions
            ->add(Crud::PAGE_INDEX, ActionDefinition::DETAIL)
            ->remove(Crud::PAGE_INDEX, ActionDefinition::EDIT);

        return parent::configureActions($actions);
    }

    public function createIndexRepositoryQuery(
            SearchDto $searchDto,
            EntityDto $entityDto,
            FieldCollection $fields,
            FilterCollection $filters
    ): RepositoryQueryInterface {
        $repositoryQuery = parent::createIndexRepositoryQuery(
                $searchDto,
                $entityDto,
                $fields,
                $filters
        );

       if(!$this->isRepositoryQueryFiltered()){

           $repositoryQuery->filterByStatus(array(
                   TicketModel::TICKET_STATUS_OPEN,
                   TicketModel::TICKET_STATUS_BEING_PROCESSED
            ));
       }

        return $repositoryQuery;
    }

    public function new(AdminContext $context)
    {
        $adminUrlGenerator = $this->get(AdminUrlGenerator::class);

        $ticket = $this->createEntity($context->getEntity()->getFqcn());

        $form = $this->createForm(TicketAdminFormType::class, $ticket);

        $form->handleRequest($context->getRequest());
        if ($form->isSubmitted() && $form->isValid()) {
            $ticket = $form->getData();

            $this->get(EntityManagerInterface::class)->create($ticket);
            $this->get(EntityManagerInterface::class)->flush();

            $url = $adminUrlGenerator
                ->setAction('index')
                ->generateUrl();

            return $this->redirect($url);
        }

        return $this->render(
            '@LcSov/admin/ticket/new.html.twig',
            [
                'form' => $form->createView(),
            ]
        );
    }

    public function detail(AdminContext $context)
    {
        $adminUrlGenerator = $this->get(AdminUrlGenerator::class);
        $ticket = $context->getEntity()->getInstance();

        $url = $adminUrlGenerator
            ->setAction('ticketStatusAction')
            ->generateUrl();

        $formTicketStatus = $this->createForm(
            TicketStatusType::class,
            $ticket,
            [
                'action' => $url,
                'method' => 'POST',
            ]
        );

        $ticketMessage = $this->get(TicketMessageContainer::class)->getFactory()->create($ticket);
        $formAddTicketMessage = $this->createForm(TicketMessageFormType::class, $ticketMessage);
        $formAddTicketMessage->handleRequest($this->get(RequestStack::class)->getMainRequest());

        if ($formAddTicketMessage->isSubmitted() && $formAddTicketMessage->isValid()) {
            $ticketMessage = $formAddTicketMessage->getData();
            $ticketMessage->setTicket($ticket);
            $ticketMessage->setAnswerByAdmin(true);
            $this->get(EntityManagerInterface::class)->create($ticketMessage);
            $this->get(EntityManagerInterface::class)->flush();
        }

        return $this->render(
            '@LcSov/admin/ticket/detail.html.twig',
            [
                'form_ticket_status' => $formTicketStatus->createView(),
                'form_add_ticket_message' => $formAddTicketMessage->createView(),
                'ticket' => $ticket,
            ]
        );
    }


    public function ticketStatusAction(AdminContext $context, EntityManagerInterface $entityManager)
    {

        $ticket = $context->getEntity()->getInstance();

        $formTicketStatusForm = $this->createForm(TicketStatusType::class, $ticket);
        $formTicketStatusForm->handleRequest($context->getRequest());

        $success = false;
        if ($formTicketStatusForm->isSubmitted() && $formTicketStatusForm->isValid()) {
            $entityManager->update($ticket);
            $entityManager->flush();
            $success = true;
        }

        return new JsonResponse(['success' => $success]);
    }

}
